﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace sou_01_interaktivni {
    public partial class Form1 : Form {
        public Form1() {
            InitializeComponent();
            loadButton_Click(null, null);
            updateTitle();
        }

        int timeLeft = 0;
        bool gameOver = true;
        int score = 0;
        bool help = false;

        int margin = 10;

        GameBoard board = new GameBoard();

        Point selected = new Point(-1, -1);

        private void myPanel1_Paint(object sender, PaintEventArgs e) {
            Graphics g = e.Graphics;

            for (int y = 0; y < 8; y++) {
                for (int x = 0; x < 8; x++) {
                    g.FillEllipse(
                        new SolidBrush(board.rows[y].tiles[x].getColor()),
                        new Rectangle(
                            PointTileToCoord(new Point(x, y)),
                            new Size(getTileSize(), getTileSize())
                        )
                    );
                    Pen penBorder = new Pen(Color.Black, 3);
                    if (selected.X == x && selected.Y == y) {
                        penBorder.Color = Color.White;
                    }
                    g.DrawEllipse(
                        penBorder,
                        new Rectangle(
                            PointTileToCoord(new Point(x, y)),
                            new Size(getTileSize(), getTileSize())
                        )
                    );
                }
            }

            for (int x = 0; x < 8; x++) {
                Pen p = new Pen(board.rows[x].getNextPreview(), 3);
                p.DashPattern = new float[] { 1.0F, 1.0F, };
                g.DrawEllipse(p,
                    new Rectangle(
                        new Point(PointTileToCoord(new Point(x, -1)).X + 3, PointTileToCoord(new Point(x, -1)).Y + 3),
                        new Size(getTileSize() - 6, getTileSize() - 6)
                    )
                );
            }

            Pen pen = new Pen(Color.White, 1);

            if (help) {
                foreach (Tuple<Point, int> t in board.turnsRemaining()) {
                    switch (t.Item2) {
                        case 0:
                            g.DrawRectangle(pen,
                                new Rectangle(
                                    PointTileToCoord(new Point(t.Item1.X, t.Item1.Y - 1)),
                                    new Size(getTileSize(), getTileSize() * 2)
                                )
                            );
                            break;
                        case 2:
                            g.DrawRectangle(pen,
                                new Rectangle(
                                    PointTileToCoord(new Point(t.Item1.X, t.Item1.Y)),
                                    new Size(getTileSize(), getTileSize() * 2)
                                )
                            );
                            break;
                        case 1:
                            g.DrawRectangle(pen,
                                new Rectangle(
                                    PointTileToCoord(new Point(t.Item1.X, t.Item1.Y)),
                                    new Size(getTileSize() * 2, getTileSize())
                                )
                            );
                            break;
                        case 3:
                            g.DrawRectangle(pen,
                                new Rectangle(
                                    PointTileToCoord(new Point(t.Item1.X - 1, t.Item1.Y)),
                                    new Size(getTileSize() * 2, getTileSize())
                                )
                            );
                            break;

                    }
                }
            }

            if (gameOver) {
                g.FillRectangle(
                    new HatchBrush(HatchStyle.Percent70, Color.Transparent, Color.White),
                    0, 0, myPanel1.Width, myPanel1.Height
                );

            }
        }

        private void myPanel1_MouseClick(object sender, MouseEventArgs e) {
            if (!gameOver) {
                Point p = PointCoordToTile(e.Location);
                if (p.X >= 0 && p.X < 9 && p.Y >= 0 && p.Y < 9) {
                    if (selected.X == -1) {
                        selected = p;
                    } else {
                        if (
                            Math.Abs(selected.X - p.X) == 1 && Math.Abs(selected.Y - p.Y) == 0 ||
                            Math.Abs(selected.X - p.X) == 0 && Math.Abs(selected.Y - p.Y) == 1
                        ) {
                            score += board.switchPlaces(selected, p);
                        }
                        selected = new Point(-1, -1);
                        chechGameEnd();
                    }
                }
            }
            myPanel1.Invalidate();
            updateTitle();
            help = false;
        }

        private void timerSec_Tick(object sender, EventArgs e) {
            timeLeft--;
            updateTitle();
            if (timeLeft <= 0) {
                timerSec.Stop();
                gameOver = true;
                myPanel1.Invalidate();
                MessageBox.Show("Čas vypršel!");
            }
        }

        private void updateTitle() {
            if (gameOver) {
                Text = "Jewels";
            } else if (timeLeft > 0) {
                Text = "Jewels - skóre " + score + " - zbývající čas " + (timeLeft / 60).ToString() + ":" + (timeLeft - (timeLeft / 60) * 60).ToString();
            } else {
                Text = "Jewels - skóre " + score;
            }
        }

        private void startLimitButton_Click(object sender, EventArgs e) {
            var splited = limitText.Text.Split(':');
            if (splited.Length == 2) {
                int min, sec;
                Int32.TryParse(splited[0], out min);
                Int32.TryParse(splited[1], out sec);
                if (min + sec > 0) {
                    timeLeft = min * 60 + sec;
                    updateTitle();
                    timerSec.Start();
                }
            }

            startGame();
        }

        private void openFile_FileOk(object sender, CancelEventArgs e) {
            if (!board.load(openFile.FileName)) {
                MessageBox.Show("Chyba souboru!");
            }
        }

        private void loadButton_Click(object sender, EventArgs e) {
            openFile.ShowDialog();
        }

        private Point PointCoordToTile(Point p) {
            return new Point(
                (p.X - getMargin(true)) / getTileSize(),
                (p.Y - getMargin(false)) / getTileSize() - 1
            );
        }

        private Point PointTileToCoord(Point p) {
            return new Point(
                getMargin(true) + p.X * getTileSize(),
                getMargin(false) + (p.Y + 1) * getTileSize()
            );
        }

        private int getMargin(bool LeftRight) {
            if (LeftRight) {
                return (myPanel1.Width - myPanel1.Height + getTileSize()) / 2;
            }
            return margin;
        }

        private int getTileSize() {
            if (myPanel1.Width < myPanel1.Height)
                return (myPanel1.Width - 2 * margin) / 8;
            return (myPanel1.Height - 2 * margin) / 9;
        }

        private void myPanel1_SizeChanged(object sender, EventArgs e) {
            myPanel1.Invalidate();
        }

        private void startNolimitButton_Click(object sender, EventArgs e) {
            startGame();
        }

        private void startGame() {
            gameOver = false;
            myPanel1.Invalidate();
            updateTitle();
        }

        private void helpButton_Click(object sender, EventArgs e) {
            help = true;
            myPanel1.Invalidate();
        }

        private void chechGameEnd() {
            if (board.turnsRemaining().Count() == 0) {
                gameOver = true;
                myPanel1.Invalidate();
                MessageBox.Show("Už nelze provést žádný tah!");
            }
        }
    }
}
