﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace sou_01_interaktivni {
    class GameBoard {
        public GameBoard() {
            rows = new List<GameBoardRow>();
            for (int i = 0; i < 8; i++) {
                rows.Add(new GameBoardRow());
            }
        }
        public List<GameBoardRow> rows { get; set; }

        public bool load(string fileName) {
            string[] fileLines = File.ReadAllLines(fileName);
            if (fileLines.Length != 16) return false;
            for (int y = 0; y < 8; y++) {
                if (fileLines[y].Length != 8) return false;
                for (int x = 0; x < 8; x++) {
                    int res;
                    Int32.TryParse(fileLines[y][x].ToString(), out res);
                    if (res < 1 || res > 6) return false;
                    rows[y].tiles[x].value = res;
                }
            }

            for (int y = 8; y < 16; y++) {
                rows[y - 8].addNext(fileLines[y]);
            }

                return true;
        }

        public bool loadForInteligence(string fileName) {
            string[] fileLines = File.ReadAllLines(fileName);
            if (fileLines.Length != 9)
                return false;
            for (int y = 0; y < 8; y++) {
                if (fileLines[y + 1].Length != 8)
                    return false;
                for (int x = 0; x < 8; x++) {
                    int res;
                    Int32.TryParse(fileLines[y][x].ToString(), out res);
                    if (res < 1 || res > 6)
                        return false;
                    rows[y].tiles[x].value = res;
                }
            }

            return true;
        }

        public int switchPlaces(Point from, Point to) {
            int val1 = rows[from.Y].tiles[from.X].value;
            int val2 = rows[to.Y].tiles[to.X].value;

            rows[from.Y].tiles[from.X].value = val2;
            rows[to.Y].tiles[to.X].value = val1;

            List<Tuple<Point, bool, int>> detected = detect();

            if (detected.Count() == 0) {
                rows[from.Y].tiles[from.X].value = val1;
                rows[to.Y].tiles[to.X].value = val2;
                return 0;
            } else {
                updateFall(detected);
                return detected.Count() * 3;
            }
        }

        private List<Tuple<Point, bool, int>> detect() {
            List<Tuple<Point, bool, int>> threes = new List<Tuple<Point, bool, int>>();
            for (int x = 1; x < 7; x++) {
                for (int y = 1; y < 7; y++) {
                    int value = rows[y].tiles[x].value;
                    if (value == 0)
                        value = -10;
                    if (value == rows[y - 1].tiles[x].value && value == rows[y + 1].tiles[x].value)
                        threes.Add(new Tuple<Point, bool, int>(new Point(x, y), false, value));
                    if (value == rows[y].tiles[x - 1].value && value == rows[y].tiles[x + 1].value)
                        threes.Add(new Tuple<Point, bool, int>(new Point(x, y), true, value));
                }
            }
            
            // První a poslední sloupec
            for (int x = 0; x < 8; x += 7) {
                for (int y = 1; y < 7; y++) {
                    int value = rows[y].tiles[x].value;
                    if (value == 0)
                        value = -10;
                    if (value == rows[y - 1].tiles[x].value && value == rows[y + 1].tiles[x].value)
                        threes.Add(new Tuple<Point, bool, int>(new Point(x, y), false, value));
                }
            }
            // První a poslední řádek
            for (int x = 1; x < 7; x++) {
                for (int y = 0; y < 8; y += 7) {
                    int value = rows[y].tiles[x].value;
                    if (value == 0)
                        value = -10;
                    if (value == rows[y].tiles[x - 1].value && value == rows[y].tiles[x + 1].value)
                        threes.Add(new Tuple<Point, bool, int>(new Point(x, y), true, value));
                }
            }
            return threes;
        }

        private void updateFall(List<Tuple<Point, bool, int>> remove) {
            foreach (Tuple<Point, bool, int> t in remove) {
                fallDown(t.Item1.X, t.Item1.Y);
                if (!t.Item2) {
                    fallDown(t.Item1.X, t.Item1.Y - 1);
                    fallDown(t.Item1.X, t.Item1.Y + 1);
                } else {
                    fallDown(t.Item1.X - 1, t.Item1.Y);
                    fallDown(t.Item1.X + 1, t.Item1.Y);
                }
            }

            List<Tuple<Point, bool, int>> detected = detect();

            if (detected.Count() != 0) updateFall(detected);
        }

        private void fallDown(int X, int Y) {
            //rows[Y].tiles[X].value = 0;
            while (Y > 0) {
                Y--;
                rows[Y + 1].tiles[X].value = rows[Y].tiles[X].value;
                rows[Y].tiles[X].value = 0;
            }
            rows[Y].tiles[X].value = rows[X].getNext();
        }

        public List<Tuple<Point, int>> turnsRemaining() {
            List<Tuple<Point, int>> canMove = new List<Tuple<Point, int>>();
            for (int x = 1; x < 7; x++) {
                for (int y = 1; y < 7; y++) {
                    int value = rows[y].tiles[x].value;
                    if (value == 0)
                        value = -10;
                    try {
                        if (
                            value == rows[y - 1].tiles[x - 1].value && value == rows[y - 2].tiles[x - 1].value ||
                            value == rows[y - 1].tiles[x - 1].value && value == rows[y + 1].tiles[x - 1].value ||
                            value == rows[y + 1].tiles[x - 1].value && value == rows[y + 2].tiles[x - 1].value
                        ) {
                            canMove.Add(new Tuple<Point, int>(new Point(x, y), 3));
                        }
                        if (
                            value == rows[y - 1].tiles[x + 1].value && value == rows[y - 2].tiles[x + 1].value ||
                            value == rows[y - 1].tiles[x + 1].value && value == rows[y + 1].tiles[x + 1].value ||
                            value == rows[y + 1].tiles[x + 1].value && value == rows[y + 2].tiles[x + 1].value
                        ) {
                            canMove.Add(new Tuple<Point, int>(new Point(x, y), 1));
                        }
                        if (
                            value == rows[y - 1].tiles[x - 1].value && value == rows[y - 1].tiles[x - 2].value ||
                            value == rows[y - 1].tiles[x - 1].value && value == rows[y - 1].tiles[x + 1].value ||
                            value == rows[y - 1].tiles[x + 1].value && value == rows[y - 1].tiles[x + 2].value
                        ) {
                            canMove.Add(new Tuple<Point, int>(new Point(x, y), 0));
                        }
                        if (
                            value == rows[y + 1].tiles[x - 1].value && value == rows[y + 1].tiles[x - 2].value ||
                            value == rows[y + 1].tiles[x - 1].value && value == rows[y + 1].tiles[x + 1].value ||
                            value == rows[y + 1].tiles[x + 1].value && value == rows[y + 1].tiles[x + 2].value) {
                            canMove.Add(new Tuple<Point, int>(new Point(x, y), 2));
                        }
                    } catch {
                        
                    }
                }
            }
            return canMove;
        }
    }
}
